@@ -816,15 +816,9 @@ static int s_default_pki_path_exists_fn(struct aws_allocator *allocator, void *c
     (void)ctx;
     (void)allocator;
 
-#        if !defined(__OpenBSD__)
-    /*
-     * OpenBSD's standard PKI directory doesn't exist on fresh installs which means this
-     * test will normally fail.
-     */
-    ASSERT_NOT_NULL(aws_determine_default_pki_dir());
-#        endif /* __OpenBSD__ */
-    ASSERT_NOT_NULL(aws_determine_default_pki_ca_file());
-
+    ASSERT_TRUE(
+        aws_determine_default_pki_dir() != NULL || aws_determine_default_pki_ca_file() != NULL,
+        "Default TLS trust store not found on this system.");
     return AWS_OP_SUCCESS;
 }
 
