@@ -364,6 +364,152 @@ static int s_test_s3_get_num_parts_and_get_part_range(struct aws_allocator *allo
     return 0;
 }
 
+struct s3_request_part_config_example {
+    const char *name;
+    uint64_t content_length;
+    size_t client_part_size;
+    uint64_t client_max_part_size;
+    size_t expected_part_size;
+    uint32_t expected_num_parts;
+};
+
+AWS_TEST_CASE(test_s3_mpu_get_part_size_and_num_parts, s_test_s3_mpu_get_part_size_and_num_parts)
+static int s_test_s3_mpu_get_part_size_and_num_parts(struct aws_allocator *allocator, void *ctx) {
+    (void)allocator;
+    (void)ctx;
+    uint64_t default_max_part_size = 5368709120ULL;
+
+    const struct s3_request_part_config_example valid_request_part_config[] = {
+        {
+            .name = "simple case",
+            .content_length = MB_TO_BYTES((uint64_t)10000),
+            .client_part_size = MB_TO_BYTES(5),
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = 5242880,
+            .expected_num_parts = 2000,
+        },
+        {
+            .name = "large content length with small part size",
+            .content_length = MB_TO_BYTES((uint64_t)990000),
+            .client_part_size = MB_TO_BYTES(5),
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = 103809024,
+            .expected_num_parts = 10000,
+        },
+        {
+
+            .name = "large content length with large part size",
+            .content_length = MB_TO_BYTES((uint64_t)1000000),
+            .client_part_size = MB_TO_BYTES(500),
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = MB_TO_BYTES(500),
+            .expected_num_parts = 2000,
+        },
+        {
+            .name = "large odd content length",
+            .content_length = 1044013645824,
+            .client_part_size = 5242880,
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = 104401365,
+            .expected_num_parts = 10000,
+        },
+        {
+            .name = "10k parts",
+            .content_length = MB_TO_BYTES((uint64_t)50000),
+            .client_part_size = MB_TO_BYTES(5),
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = MB_TO_BYTES(5),
+            .expected_num_parts = 10000,
+        },
+        {
+            .name = "10k - 1 parts",
+            .content_length = 49995,
+            .client_part_size = 5,
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = 5,
+            .expected_num_parts = 9999,
+        },
+        {
+            .name = "10k with small last part",
+            .content_length = 49998,
+            .client_part_size = 5,
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = 5,
+            .expected_num_parts = 10000,
+        },
+        {
+            .name = "10k + 1 parts",
+            .content_length = 50001,
+            .client_part_size = 5,
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = 6,
+            .expected_num_parts = 8334,
+
+        },
+        {
+            .name = "bump content length",
+            .content_length = 100000,
+            .client_part_size = 5,
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = 10,
+            .expected_num_parts = 10000,
+        },
+        {
+            .name = "bump content length with non-zero mod",
+            .content_length = 999999,
+            .client_part_size = 5,
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = 100,
+            .expected_num_parts = 10000,
+        },
+        {
+            .name = "5 tb content length",
+            .content_length = MB_TO_BYTES((uint64_t)5 * 1024 * 1024),
+            .client_part_size = MB_TO_BYTES((uint64_t)5),
+            .client_max_part_size = default_max_part_size,
+            .expected_part_size = 549755814,
+            .expected_num_parts = 10000,
+        },
+    };
+    for (size_t i = 0; i < AWS_ARRAY_SIZE(valid_request_part_config); ++i) {
+        AWS_LOGF_INFO(AWS_LS_S3_GENERAL, "valid example [%zu]: %s\n", i, valid_request_part_config[i].name);
+
+        uint64_t content_length = valid_request_part_config[i].content_length;
+        size_t part_size;
+        uint32_t num_parts;
+
+        ASSERT_SUCCESS(aws_s3_calculate_optimal_mpu_part_size_and_num_parts(
+            content_length,
+            valid_request_part_config[i].client_part_size,
+            valid_request_part_config[i].client_max_part_size,
+            &part_size,
+            &num_parts));
+        ASSERT_INT_EQUALS(valid_request_part_config[i].expected_part_size, part_size);
+        ASSERT_INT_EQUALS(valid_request_part_config[i].expected_num_parts, num_parts);
+    }
+
+    /* Invalid cases */
+    const struct s3_request_part_config_example invalid_request_part_config[] = {{
+        .name = "max part < required part size",
+        .content_length = 900000,
+        .client_part_size = 5,
+        .client_max_part_size = 10,
+    }};
+
+    for (size_t i = 0; i < AWS_ARRAY_SIZE(invalid_request_part_config); ++i) {
+        printf("invalid example [%zu]: %s\n", i, invalid_request_part_config[i].name);
+        size_t part_size;
+        uint32_t num_parts;
+        ASSERT_FAILS(aws_s3_calculate_optimal_mpu_part_size_and_num_parts(
+            invalid_request_part_config[i].content_length,
+            invalid_request_part_config[i].client_part_size,
+            invalid_request_part_config[i].client_max_part_size,
+            &part_size,
+            &num_parts));
+    }
+    return AWS_OP_SUCCESS;
+}
+
 AWS_TEST_CASE(test_s3_aws_xml_get_top_level_tag_with_root_name, s_test_s3_aws_xml_get_top_level_tag_with_root_name)
 static int s_test_s3_aws_xml_get_top_level_tag_with_root_name(struct aws_allocator *allocator, void *ctx) {
     (void)allocator;
