@@ -60,14 +60,24 @@
 #include <stdlib.h>
 
 #include <openssl/base.h>
+#include <openssl/cipher.h>
 #include <openssl/type_check.h>
 
 #include "../internal.h"
+#include "../fipsmodule/cpucap/internal.h"
 
 #if defined(__cplusplus)
 extern "C" {
 #endif
 
+#if !defined(OPENSSL_NO_ASM) && defined(OPENSSL_X86_64) && \
+    !defined(MY_ASSEMBLER_IS_TOO_OLD_FOR_AVX) && !defined(AWSLC_FIPS)
+#define AES_CBC_HMAC_SHA_STITCH
+// TLS1_1_VERSION is also defined in ssl.h.
+#define TLS1_1_VERSION 0x0302
+#define NO_PAYLOAD_LENGTH ((size_t)-1)
+#define HMAC_KEY_SIZE 64
+#endif
 
 // EVP_tls_cbc_get_padding determines the padding from the decrypted, TLS, CBC
 // record in |in|. This decrypted record should not include any "decrypted"
@@ -98,17 +108,22 @@ void EVP_tls_cbc_copy_mac(uint8_t *out, size_t md_size, const uint8_t *in,
 // which EVP_tls_cbc_digest_record supports.
 int EVP_tls_cbc_record_digest_supported(const EVP_MD *md);
 
-// EVP_sha1_final_with_secret_suffix computes the result of hashing |len| bytes
-// from |in| to |ctx| and writes the resulting hash to |out|. |len| is treated
-// as secret and must be at most |max_len|, which is treated as public. |in|
-// must point to a buffer of at least |max_len| bytes. It returns one on success
-// and zero if inputs are too long.
+// EVP_final_with_secret_suffix_sha1 and EVP_final_with_secret_suffix_sha256
+// compute the result of hashing |len| bytes from |in| to |ctx| and write the
+// resulting hash to |out|. |len| is treated as secret and must be at most
+// |max_len|, which is treated as public. |in| must point to a buffer of at
+// least |max_len| bytes. It returns one on success and zero if inputs are
+// too long.
 //
-// This function is exported for unit tests.
-OPENSSL_EXPORT int EVP_sha1_final_with_secret_suffix(
+// The functions are exported for unit tests.
+OPENSSL_EXPORT int EVP_final_with_secret_suffix_sha1(
     SHA_CTX *ctx, uint8_t out[SHA_DIGEST_LENGTH], const uint8_t *in, size_t len,
     size_t max_len);
 
+OPENSSL_EXPORT int EVP_final_with_secret_suffix_sha256(
+    SHA256_CTX *ctx, uint8_t out[SHA256_DIGEST_LENGTH], const uint8_t *in,
+    size_t len, size_t max_len);
+
 // EVP_tls_cbc_digest_record computes the MAC of a decrypted, padded TLS
 // record.
 //
@@ -134,6 +149,15 @@ int EVP_tls_cbc_digest_record(const EVP_MD *md, uint8_t *md_out,
                               const uint8_t *mac_secret,
                               unsigned mac_secret_length);
 
+// EVP_tls_cbc_digest_record_sha256 performs the same functionality of 
+// EVP_tls_cbc_digest_record except it internally calls SHA256 instead of SHA1.
+int EVP_tls_cbc_digest_record_sha256(const EVP_MD *md, uint8_t *md_out,
+                              size_t *md_out_size, const uint8_t header[13],
+                              const uint8_t *data, size_t data_size,
+                              size_t data_plus_mac_plus_padding_size,
+                              const uint8_t *mac_secret,
+                              unsigned mac_secret_length);
+
 #define POLY1305_TAG_LEN 16
 
 // For convenience (the x86_64 calling convention allows only six parameters in
@@ -173,14 +197,13 @@ OPENSSL_STATIC_ASSERT(sizeof(union chacha20_poly1305_seal_data) == 48 + 8 + 8,
 
 OPENSSL_INLINE int chacha20_poly1305_asm_capable(void) {
 #if defined(OPENSSL_X86_64)
-  const int sse41_capable = (OPENSSL_ia32cap_P[1] & (1 << 19)) != 0;
-  return sse41_capable;
+  return CRYPTO_is_SSE4_1_capable();
 #elif defined(OPENSSL_AARCH64)
   return CRYPTO_is_NEON_capable();
 #endif
 }
 
-// chacha20_poly1305_open is defined in chacha20_poly1305_x86_64.pl. It decrypts
+// chacha20_poly1305_open is defined in chacha20_poly1305_*.pl. It decrypts
 // |plaintext_len| bytes from |ciphertext| and writes them to |out_plaintext|.
 // Additional input parameters are passed in |aead_data->in|. On exit, it will
 // write calculated tag value to |aead_data->out.tag|, which the caller must
@@ -191,7 +214,7 @@ extern void chacha20_poly1305_open(uint8_t *out_plaintext,
                                    size_t ad_len,
                                    union chacha20_poly1305_open_data *data);
 
-// chacha20_poly1305_open is defined in chacha20_poly1305_x86_64.pl. It encrypts
+// chacha20_poly1305_open is defined in chacha20_poly1305_*.pl. It encrypts
 // |plaintext_len| bytes from |plaintext| and writes them to |out_ciphertext|.
 // Additional input parameters are passed in |aead_data->in|. The calculated tag
 // value is over the computed ciphertext concatenated with |extra_ciphertext|
