@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: Apache-2.0 OR ISC
 
 #include <openssl/base.h>
+#include "../fipsmodule/evp/internal.h"
 
 typedef struct {
   // key is the concatenation of the private seed and public key. It is stored
@@ -22,17 +23,31 @@ typedef struct {
   char has_private;
 } X25519_KEY;
 
+#ifdef ENABLE_DILITHIUM
+
+typedef struct {
+  uint8_t pub[1952];
+  uint8_t priv[4000];
+  char has_private;
+} DILITHIUM3_KEY;
+
+#endif
+
 extern const EVP_PKEY_ASN1_METHOD dsa_asn1_meth;
 extern const EVP_PKEY_ASN1_METHOD ec_asn1_meth;
 extern const EVP_PKEY_ASN1_METHOD rsa_asn1_meth;
 extern const EVP_PKEY_ASN1_METHOD rsa_pss_asn1_meth;
 extern const EVP_PKEY_ASN1_METHOD ed25519_asn1_meth;
 extern const EVP_PKEY_ASN1_METHOD x25519_asn1_meth;
+#ifdef ENABLE_DILITHIUM
+extern const EVP_PKEY_ASN1_METHOD dilithium3_asn1_meth;
+#endif
 extern const EVP_PKEY_ASN1_METHOD kem_asn1_meth;
 
 extern const EVP_PKEY_METHOD ed25519_pkey_meth;
 extern const EVP_PKEY_METHOD x25519_pkey_meth;
 extern const EVP_PKEY_METHOD hkdf_pkey_meth;
+extern const EVP_PKEY_METHOD dilithium3_pkey_meth;
 extern const EVP_PKEY_METHOD kem_pkey_meth;
 
 // Returns a reference to the list |non_fips_pkey_evp_methods|. The list has
