@@ -105,10 +105,6 @@ TEST(ForkDetect, Test) {
     CRYPTO_fork_detect_ignore_madv_wipeonfork_for_testing();
   }
 
-  if (getenv("BORINGSSL_IGNORE_PTHREAD_ATFORK")) {
-    CRYPTO_fork_detect_ignore_pthread_atfork_for_testing();
-  }
-
   const uint64_t start = CRYPTO_get_fork_generation();
   if (start == 0) {
     fprintf(stderr, "Fork detection not supported. Skipping test.\n");
