@@ -328,15 +328,27 @@ OPENSSL_EXPORT int i2d_DHparams(const DH *in, unsigned char **outp);
 OPENSSL_EXPORT int DH_compute_key(uint8_t *out, const BIGNUM *peers_key,
                                   DH *dh);
 
-// Standard parameters. These parameters are taken from RFC 5114.
-// This function returns a new DH object with standard parameters. It returns
-// NULL on allocation failure.
+// DH_get_2048_256 returns the 2048-bit MODP Group with 256-bit Prime Order
+// Subgroup from RFC 5114. This function returns a new DH object with standard
+// parameters. It returns NULL on allocation failure.
 //
 // Warning: 2048-256 is no longer an optimal parameter for Diffie-Hellman. No
 // one should use finite field Diffie-Hellman anymore.
 // This function has been deprecated with no replacement.
 OPENSSL_EXPORT DH *DH_get_2048_256(void);
 
+// DH_clear_flags does nothing and is included to simplify compiling code that
+// expects it.
+OPENSSL_EXPORT void DH_clear_flags(DH *dh, int flags);
+
+// DH_FLAG_CACHE_MONT_P is not supported by AWS-LC and is included to simplify
+// compiling code that expects it. This flag controls if the DH APIs should
+// cache the montgomery form of the prime to speed up multiplication at the cost
+// of increasing memory storage. AWS-LC always does this and does not support
+// turning this option off.
+#define DH_FLAG_CACHE_MONT_P 0
+
+
 #if defined(__cplusplus)
 }  // extern C
 
