@@ -197,20 +197,26 @@ impl<'a, T: 'a + Context> Callback<'a, T> {
 
 #[cfg(test)]
 mod tests {
-    use crate::testing::*;
+    use crate::{
+        callbacks::{ClientHelloCallback, ConnectionFuture},
+        enums::ClientAuthType,
+        testing::{client_hello::*, s2n_tls::*, *},
+    };
+    use alloc::sync::Arc;
+    use core::sync::atomic::Ordering;
     use futures_test::task::new_count_waker;
-    use std::{fs, path::Path};
+    use std::{fs, path::Path, pin::Pin, sync::atomic::AtomicUsize};
 
     #[test]
     fn handshake_default() {
         let config = build_config(&security::DEFAULT).unwrap();
-        s2n_tls_pair(config);
+        establish_connection(config);
     }
 
     #[test]
     fn handshake_default_tls13() {
         let config = build_config(&security::DEFAULT_TLS13).unwrap();
-        s2n_tls_pair(config)
+        establish_connection(config)
     }
 
     #[test]
@@ -292,6 +298,107 @@ mod tests {
         assert_eq!(wake_count, 0);
     }
 
+    #[test]
+    fn failing_client_hello_callback_sync() -> Result<(), Error> {
+        let (waker, wake_count) = new_count_waker();
+        let handle = FailingCHHandler::default();
+        let config = {
+            let mut config = config_builder(&security::DEFAULT_TLS13)?;
+            config.set_client_hello_callback(handle)?;
+            config.build()?
+        };
+
+        let server = {
+            // create and configure a server connection
+            let mut server = crate::connection::Connection::new_server();
+            server.set_config(config.clone())?;
+            server.set_waker(Some(&waker))?;
+            Harness::new(server)
+        };
+
+        let client = {
+            // create a client connection
+            let mut client = crate::connection::Connection::new_client();
+            client.set_config(config)?;
+            Harness::new(client)
+        };
+
+        let mut pair = Pair::new(server, client, SAMPLES);
+        loop {
+            match pair.poll() {
+                Poll::Ready(result) => {
+                    let err = result.expect_err("handshake should fail");
+
+                    // the underlying error should be the custom error the application provided
+                    let s2n_err = err.downcast_ref::<crate::error::Error>().unwrap();
+                    let app_err = s2n_err.application_error().unwrap();
+                    let io_err = app_err.downcast_ref::<std::io::Error>().unwrap();
+                    let _custom_err = io_err
+                        .get_ref()
+                        .unwrap()
+                        .downcast_ref::<CustomError>()
+                        .unwrap();
+                    break;
+                }
+                Poll::Pending => continue,
+            }
+        }
+        assert_eq!(wake_count, 0);
+
+        Ok(())
+    }
+
+    #[test]
+    fn failing_client_hello_callback_async() -> Result<(), Error> {
+        let (waker, wake_count) = new_count_waker();
+        let handle = FailingAsyncCHHandler::default();
+        let config = {
+            let mut config = config_builder(&security::DEFAULT_TLS13)?;
+            config.set_client_hello_callback(handle)?;
+            config.build()?
+        };
+
+        let server = {
+            // create and configure a server connection
+            let mut server = crate::connection::Connection::new_server();
+            server.set_config(config.clone())?;
+            server.set_waker(Some(&waker))?;
+            Harness::new(server)
+        };
+
+        let client = {
+            // create a client connection
+            let mut client = crate::connection::Connection::new_client();
+            client.set_config(config)?;
+            Harness::new(client)
+        };
+
+        let mut pair = Pair::new(server, client, SAMPLES);
+        loop {
+            match pair.poll() {
+                Poll::Ready(result) => {
+                    let err = result.expect_err("handshake should fail");
+
+                    // the underlying error should be the custom error the application provided
+                    let s2n_err = err.downcast_ref::<crate::error::Error>().unwrap();
+                    let app_err = s2n_err.application_error().unwrap();
+                    let io_err = app_err.downcast_ref::<std::io::Error>().unwrap();
+                    let _custom_err = io_err
+                        .get_ref()
+                        .unwrap()
+                        .downcast_ref::<CustomError>()
+                        .unwrap();
+                    break;
+                }
+                Poll::Pending => continue,
+            }
+        }
+        // assert that the future is async returned Poll::Pending once
+        assert_eq!(wake_count, 1);
+
+        Ok(())
+    }
+
     #[test]
     fn client_hello_callback_async() -> Result<(), Error> {
         let (waker, wake_count) = new_count_waker();
@@ -330,11 +437,12 @@ mod tests {
             handle.invoked.load(Ordering::SeqCst),
             require_pending_count + 1
         );
+
         Ok(())
     }
-
     #[test]
     fn client_hello_callback_sync() -> Result<(), Error> {
+        let (waker, wake_count) = new_count_waker();
         #[derive(Clone)]
         struct ClientHelloSyncCallback(Arc<AtomicUsize>);
         impl ClientHelloSyncCallback {
@@ -346,10 +454,10 @@ mod tests {
             }
         }
         impl ClientHelloCallback for ClientHelloSyncCallback {
-            fn poll_client_hello(
+            fn on_client_hello(
                 &self,
                 connection: &mut crate::connection::Connection,
-            ) -> Poll<Result<(), error::Error>> {
+            ) -> Result<Option<Pin<Box<dyn ConnectionFuture>>>, crate::error::Error> {
                 // Test that the config can be changed
                 connection
                     .set_config(build_config(&security::DEFAULT_TLS13).unwrap())
@@ -359,7 +467,10 @@ mod tests {
                 connection.server_name_extension_used();
 
                 self.0.fetch_add(1, Ordering::Relaxed);
-                Poll::Ready(Ok(()))
+
+                // returning `None` indicates that the client_hello callback is
+                // finished and the handshake can proceed.
+                Ok(None)
             }
         }
         let callback = ClientHelloSyncCallback::new();
@@ -374,6 +485,7 @@ mod tests {
             // create and configure a server connection
             let mut server = crate::connection::Connection::new_server();
             server.set_config(config.clone())?;
+            server.set_waker(Some(&waker))?;
             Harness::new(server)
         };
 
@@ -389,6 +501,7 @@ mod tests {
         assert_eq!(callback.count(), 0);
         poll_tls_pair(pair);
         assert_eq!(callback.count(), 1);
+        assert_eq!(wake_count, 0);
         Ok(())
     }
 
@@ -403,13 +516,6 @@ mod tests {
 
     #[test]
     fn trust_location() -> Result<(), Error> {
-        struct AcceptAllHostnames {}
-        impl VerifyHostNameCallback for AcceptAllHostnames {
-            fn verify_host_name(&self, _: &str) -> bool {
-                true
-            }
-        }
-
         let pem_dir = Path::new(concat!(env!("CARGO_MANIFEST_DIR"), "/../../../tests/pems"));
         let mut cert = pem_dir.to_path_buf();
         cert.push("rsa_4096_sha512_client_cert.pem");
@@ -418,11 +524,87 @@ mod tests {
 
         let mut builder = crate::config::Builder::new();
         builder.set_security_policy(&security::DEFAULT_TLS13)?;
-        builder.set_verify_host_callback(AcceptAllHostnames {})?;
+        builder.set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})?;
         builder.load_pem(&fs::read(&cert)?, &fs::read(&key)?)?;
         builder.trust_location(Some(&cert), None)?;
 
-        s2n_tls_pair(builder.build()?);
+        establish_connection(builder.build()?);
+        Ok(())
+    }
+
+    #[test]
+    fn connection_level_verify_host_callback() -> Result<(), Error> {
+        let reject_config = {
+            let mut keypair = CertKeyPair::default();
+            let mut config = crate::config::Builder::new();
+            // configure the config VerifyHostNameCallback to reject all certificates
+            config.set_verify_host_callback(RejectAllCertificatesHandler {})?;
+            config.set_security_policy(&security::DEFAULT_TLS13)?;
+            config.load_pem(keypair.cert(), keypair.key())?;
+            config.trust_pem(keypair.cert())?;
+            config.set_client_auth_type(ClientAuthType::Required)?;
+            config.build()?
+        };
+
+        // confirm that default connection establishment fails
+        let mut pair = tls_pair(reject_config.clone());
+        assert!(poll_tls_pair_result(&mut pair).is_err());
+
+        // confirm that overriding the verify_host_callback on connection causes
+        // the handshake to succeed
+        pair = tls_pair(reject_config);
+        pair.server
+            .0
+            .connection
+            .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})
+            .unwrap();
+        pair.client
+            .0
+            .connection
+            .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})
+            .unwrap();
+        assert!(poll_tls_pair_result(&mut pair).is_ok());
+
+        Ok(())
+    }
+
+    #[test]
+    fn no_client_auth() -> Result<(), Error> {
+        use crate::enums::ClientAuthType;
+
+        let config = {
+            let mut config = config_builder(&security::DEFAULT_TLS13)?;
+            config.set_client_auth_type(ClientAuthType::None)?;
+            config.build()?
+        };
+
+        let server = {
+            let mut server = crate::connection::Connection::new_server();
+            server.set_config(config.clone())?;
+            Harness::new(server)
+        };
+
+        let client = {
+            let mut client = crate::connection::Connection::new_client();
+            client.set_config(config)?;
+            Harness::new(client)
+        };
+
+        let pair = Pair::new(server, client, SAMPLES);
+        let pair = poll_tls_pair(pair);
+        let server = pair.server.0.connection;
+        let client = pair.client.0.connection;
+
+        for conn in [server, client] {
+            assert!(!conn.client_cert_used());
+            let cert = conn.client_cert_chain_bytes()?;
+            assert!(cert.is_none());
+            let sig_alg = conn.selected_client_signature_algorithm()?;
+            assert!(sig_alg.is_none());
+            let hash_alg = conn.selected_client_hash_algorithm()?;
+            assert!(hash_alg.is_none());
+        }
+
         Ok(())
     }
 
@@ -453,12 +635,18 @@ mod tests {
         let server = pair.server.0.connection;
         let client = pair.client.0.connection;
 
-        assert!(server.client_cert_used());
-        assert!(client.client_cert_used());
         let cert = server.client_cert_chain_bytes()?;
         assert!(cert.is_some());
         assert!(!cert.unwrap().is_empty());
 
+        for conn in [server, client] {
+            assert!(conn.client_cert_used());
+            let sig_alg = conn.selected_client_signature_algorithm()?;
+            assert!(sig_alg.is_some());
+            let hash_alg = conn.selected_client_hash_algorithm()?;
+            assert!(hash_alg.is_some());
+        }
+
         Ok(())
     }
 }
