@@ -14,36 +14,31 @@
  */
 
 #include <sys/param.h>
-
-#include "api/s2n.h"
 #include <time.h>
 
+#include "api/s2n.h"
 #include "crypto/s2n_fips.h"
-
 #include "error/s2n_errno.h"
-
+#include "stuffer/s2n_stuffer.h"
+#include "tls/s2n_alerts.h"
 #include "tls/s2n_cipher_preferences.h"
 #include "tls/s2n_cipher_suites.h"
 #include "tls/s2n_connection.h"
-#include "tls/s2n_alerts.h"
+#include "tls/s2n_security_policies.h"
 #include "tls/s2n_server_extensions.h"
 #include "tls/s2n_tls.h"
 #include "tls/s2n_tls13.h"
-#include "tls/s2n_security_policies.h"
 #include "tls/s2n_tls13_handshake.h"
 #include "tls/s2n_tls13_key_schedule.h"
-
-#include "stuffer/s2n_stuffer.h"
-
-#include "utils/s2n_safety.h"
-#include "utils/s2n_random.h"
 #include "utils/s2n_bitmap.h"
+#include "utils/s2n_random.h"
+#include "utils/s2n_safety.h"
 
 /* From RFC5246 7.4.1.2. */
 #define S2N_TLS_COMPRESSION_METHOD_NULL 0
 
 /* From RFC8446 4.1.3. */
-#define S2N_DOWNGRADE_PROTECTION_SIZE   8
+#define S2N_DOWNGRADE_PROTECTION_SIZE 8
 const uint8_t tls12_downgrade_protection_bytes[] = {
     0x44, 0x4F, 0x57, 0x4E, 0x47, 0x52, 0x44, 0x01
 };
@@ -52,16 +47,18 @@ const uint8_t tls11_downgrade_protection_bytes[] = {
     0x44, 0x4F, 0x57, 0x4E, 0x47, 0x52, 0x44, 0x00
 };
 
-static int s2n_random_value_is_hello_retry(struct s2n_connection *conn) {
+static int s2n_random_value_is_hello_retry(struct s2n_connection *conn)
+{
     POSIX_ENSURE_REF(conn);
 
     POSIX_ENSURE(memcmp(hello_retry_req_random, conn->handshake_params.server_random, S2N_TLS_RANDOM_DATA_LEN) == 0,
-                 S2N_ERR_INVALID_HELLO_RETRY);
+            S2N_ERR_INVALID_HELLO_RETRY);
 
     return S2N_SUCCESS;
 }
 
-static int s2n_client_detect_downgrade_mechanism(struct s2n_connection *conn) {
+static int s2n_client_detect_downgrade_mechanism(struct s2n_connection *conn)
+{
     POSIX_ENSURE_REF(conn);
     uint8_t *downgrade_bytes = &conn->handshake_params.server_random[S2N_TLS_RANDOM_DATA_LEN - S2N_DOWNGRADE_PROTECTION_SIZE];
 
@@ -79,7 +76,8 @@ static int s2n_client_detect_downgrade_mechanism(struct s2n_connection *conn) {
     return 0;
 }
 
-static int s2n_server_add_downgrade_mechanism(struct s2n_connection *conn) {
+static int s2n_server_add_downgrade_mechanism(struct s2n_connection *conn)
+{
     POSIX_ENSURE_REF(conn);
     uint8_t *downgrade_bytes = &conn->handshake_params.server_random[S2N_TLS_RANDOM_DATA_LEN - S2N_DOWNGRADE_PROTECTION_SIZE];
 
@@ -109,7 +107,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
     POSIX_GUARD(s2n_stuffer_read_bytes(in, protocol_version, S2N_TLS_PROTOCOL_VERSION_LEN));
     POSIX_GUARD(s2n_stuffer_read_bytes(in, conn->handshake_params.server_random, S2N_TLS_RANDOM_DATA_LEN));
 
-    uint8_t legacy_version = (uint8_t)(protocol_version[0] * 10) + protocol_version[1];
+    uint8_t legacy_version = (uint8_t) (protocol_version[0] * 10) + protocol_version[1];
 
     /**
      *= https://tools.ietf.org/rfc/rfc8446#4.1.3
@@ -118,7 +116,6 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
      *# it as described in Section 4.1.4).
      **/
     if (s2n_random_value_is_hello_retry(conn) == S2N_SUCCESS) {
-
         /**
          *= https://tools.ietf.org/rfc/rfc8446#4.1.4
          *# If a client receives a second
@@ -160,7 +157,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
     S2N_ERROR_IF(compression_method != S2N_TLS_COMPRESSION_METHOD_NULL, S2N_ERR_BAD_MESSAGE);
 
     bool session_ids_match = session_id_len != 0 && session_id_len == conn->session_id_len
-                && memcmp(session_id, conn->session_id, session_id_len) == 0;
+            && memcmp(session_id, conn->session_id, session_id_len) == 0;
     if (!session_ids_match) {
         conn->ems_negotiated = false;
     }
@@ -175,7 +172,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
         s2n_extension_type_id supported_versions_id = s2n_unsupported_extension;
         POSIX_GUARD(s2n_extension_supported_iana_value_to_id(TLS_EXTENSION_SUPPORTED_VERSIONS, &supported_versions_id));
         POSIX_ENSURE(S2N_CBIT_TEST(conn->extension_responses_received, supported_versions_id),
-                     S2N_ERR_MISSING_EXTENSION);
+                S2N_ERR_MISSING_EXTENSION);
     }
 
     if (conn->server_protocol_version >= S2N_TLS13) {
@@ -221,7 +218,7 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
             POSIX_BAIL(S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED);
         }
 
-        uint8_t actual_protocol_version = MIN(conn->server_protocol_version, conn->client_protocol_version);
+        conn->actual_protocol_version = MIN(conn->server_protocol_version, conn->client_protocol_version);
 
         /*
          *= https://tools.ietf.org/rfc/rfc5077#section-3.4
@@ -233,18 +230,18 @@ static int s2n_server_hello_parse(struct s2n_connection *conn)
          */
         if (session_ids_match) {
             /* check if the resumed session state is valid */
-            S2N_ERROR_IF(conn->actual_protocol_version != actual_protocol_version, S2N_ERR_BAD_MESSAGE);
-            S2N_ERROR_IF(memcmp(conn->secure->cipher_suite->iana_value, cipher_suite_wire, S2N_TLS_CIPHER_SUITE_LEN) != 0, S2N_ERR_BAD_MESSAGE);
+            POSIX_ENSURE(conn->resume_protocol_version == conn->actual_protocol_version, S2N_ERR_BAD_MESSAGE);
+            POSIX_ENSURE(memcmp(conn->secure->cipher_suite->iana_value, cipher_suite_wire, S2N_TLS_CIPHER_SUITE_LEN) == 0,
+                    S2N_ERR_BAD_MESSAGE);
 
             /* Session is resumed */
             conn->client_session_resumed = 1;
         } else {
             conn->session_id_len = session_id_len;
             POSIX_CHECKED_MEMCPY(conn->session_id, session_id, session_id_len);
-            conn->actual_protocol_version = actual_protocol_version;
             POSIX_GUARD(s2n_set_cipher_as_client(conn, cipher_suite_wire));
             /* Erase master secret which might have been set for session resumption */
-            POSIX_CHECKED_MEMSET((uint8_t *)conn->secrets.tls12.master_secret, 0, S2N_TLS_SECRET_LEN);
+            POSIX_CHECKED_MEMSET((uint8_t *) conn->secrets.version.tls12.master_secret, 0, S2N_TLS_SECRET_LEN);
 
             /* Erase client session ticket which might have been set for session resumption */
             POSIX_GUARD(s2n_free(&conn->client_ticket));
@@ -305,8 +302,8 @@ int s2n_server_hello_write_message(struct s2n_connection *conn)
      * https://tools.ietf.org/html/rfc8446#section-4.1.3 */
     const uint16_t legacy_protocol_version = MIN(conn->actual_protocol_version, S2N_TLS12);
     uint8_t protocol_version[S2N_TLS_PROTOCOL_VERSION_LEN];
-    protocol_version[0] = (uint8_t)(legacy_protocol_version / 10);
-    protocol_version[1] = (uint8_t)(legacy_protocol_version % 10);
+    protocol_version[0] = (uint8_t) (legacy_protocol_version / 10);
+    protocol_version[1] = (uint8_t) (legacy_protocol_version % 10);
 
     POSIX_GUARD(s2n_stuffer_write_bytes(&conn->handshake.io, protocol_version, S2N_TLS_PROTOCOL_VERSION_LEN));
     POSIX_GUARD(s2n_stuffer_write_bytes(&conn->handshake.io, conn->handshake_params.server_random, S2N_TLS_RANDOM_DATA_LEN));
@@ -322,14 +319,14 @@ int s2n_server_hello_send(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
 
-    struct s2n_stuffer server_random = {0};
-    struct s2n_blob b = {0};
+    struct s2n_stuffer server_random = { 0 };
+    struct s2n_blob b = { 0 };
     POSIX_GUARD(s2n_blob_init(&b, conn->handshake_params.server_random, S2N_TLS_RANDOM_DATA_LEN));
 
     /* Create the server random data */
     POSIX_GUARD(s2n_stuffer_init(&server_random, &b));
 
-    struct s2n_blob rand_data = {0};
+    struct s2n_blob rand_data = { 0 };
     POSIX_GUARD(s2n_blob_init(&rand_data, s2n_stuffer_raw_write(&server_random, S2N_TLS_RANDOM_DATA_LEN), S2N_TLS_RANDOM_DATA_LEN));
     POSIX_ENSURE_REF(rand_data.data);
     POSIX_GUARD_RESULT(s2n_get_public_random_data(&rand_data));
