@@ -9,6 +9,7 @@ import os
 import sys
 from test import NativeResourceTest, TIMEOUT
 import time
+import unittest
 
 EXAMPLE_ACCESS_KEY_ID = 'example_access_key_id'
 EXAMPLE_SECRET_ACCESS_KEY = 'example_secret_access_key'
@@ -220,6 +221,86 @@ class TestProvider(NativeResourceTest):
             credentials = credentials_future.result(TIMEOUT)
 
 
+@unittest.skipUnless(os.environ.get('AWS_TEST_MQTT311_COGNITO_IDENTITY') and os.environ.get('AWS_TEST_MQTT311_COGNITO_ENDPOINT'),
+                     'set env var to run test: AWS_TEST_MQTT311_COGNITO_IDENTITY and AWS_TEST_MQTT311_COGNITO_ENDPOINT')
+class CognitoCredentialsProviderTest(NativeResourceTest):
+
+    def test_unauthenticated(self):
+        identity = os.environ.get('AWS_TEST_MQTT311_COGNITO_IDENTITY')
+        identity_endpoint = os.environ.get('AWS_TEST_MQTT311_COGNITO_ENDPOINT')
+        tls_opts = awscrt.io.TlsContextOptions()
+        tls_context = awscrt.io.ClientTlsContext(tls_opts)
+
+        provider = awscrt.auth.AwsCredentialsProvider.new_cognito(
+            endpoint=identity_endpoint,
+            identity=identity,
+            tls_ctx=tls_context
+        )
+
+        self.assertIsNotNone(provider)
+        credentials = provider.get_credentials().result(TIMEOUT)
+        self.assertIsNotNone(credentials)
+
+    def test_cognito_provider_create_exception_bad_login(self):
+        identity = os.environ.get('AWS_TEST_MQTT311_COGNITO_IDENTITY')
+        identity_endpoint = os.environ.get('AWS_TEST_MQTT311_COGNITO_ENDPOINT')
+        tls_opts = awscrt.io.TlsContextOptions()
+        tls_context = awscrt.io.ClientTlsContext(tls_opts)
+
+        with self.assertRaises(Exception):
+            provider = awscrt.auth.AwsCredentialsProvider.new_cognito(
+                endpoint=identity_endpoint,
+                identity=identity,
+                tls_ctx=tls_context,
+                logins=[('provider1', 5), ('provider2', ['List not string'])]
+            )
+
+            self.assertIsNone(provider)
+
+    def test_maximal_create(self):
+        identity = os.environ.get('AWS_TEST_MQTT311_COGNITO_IDENTITY')
+        identity_endpoint = os.environ.get('AWS_TEST_MQTT311_COGNITO_ENDPOINT')
+        tls_opts = awscrt.io.TlsContextOptions()
+        tls_context = awscrt.io.ClientTlsContext(tls_opts)
+
+        provider = awscrt.auth.AwsCredentialsProvider.new_cognito(
+            endpoint=identity_endpoint,
+            identity=identity,
+            tls_ctx=tls_context,
+            logins=[('provider1', 'token1'), ('provide2', 'token2')],
+            custom_role_arn='not-a-real-arn'
+        )
+
+        self.assertIsNotNone(provider)
+
+
+@unittest.skipUnless(os.environ.get('AWS_TESTING_COGNITO_IDENTITY') and os.environ.get('AWS_TEST_MQTT311_COGNITO_ENDPOINT')
+                     and os.environ.get('AWS_TEST_HTTP_PROXY_HOST') and os.environ.get('AWS_TEST_HTTP_PROXY_PORT'),
+                     'set env var to run test: AWS_TESTING_COGNITO_IDENTITY, AWS_TEST_MQTT311_COGNITO_ENDPOINT, '
+                     'AWS_TEST_HTTP_PROXY_HOST, and AWS_TEST_HTTP_PROXY_PORT')
+class CognitoCredentialsProviderProxyTest(NativeResourceTest):
+
+    def test_unauthenticated(self):
+        identity = os.environ.get('AWS_TESTING_COGNITO_IDENTITY')
+        identity_endpoint = os.environ.get('AWS_TEST_MQTT311_COGNITO_ENDPOINT')
+        tls_opts = awscrt.io.TlsContextOptions()
+        tls_context = awscrt.io.ClientTlsContext(tls_opts)
+        http_proxy_options = awscrt.http.HttpProxyOptions(
+            os.environ.get('AWS_TEST_HTTP_PROXY_HOST'),
+            int(os.environ.get('AWS_TEST_HTTP_PROXY_PORT', '0')))
+
+        provider = awscrt.auth.AwsCredentialsProvider.new_cognito(
+            endpoint=identity_endpoint,
+            identity=identity,
+            tls_ctx=tls_context,
+            http_proxy_options=http_proxy_options
+        )
+
+        self.assertIsNotNone(provider)
+        credentials = provider.get_credentials().result(TIMEOUT)
+        self.assertIsNotNone(credentials)
+
+
 class TestSigningConfig(NativeResourceTest):
     def test_create(self):
         algorithm = awscrt.auth.AwsSigningAlgorithm.V4
